@@ -87,6 +87,7 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
             pub use checkout_core::error::new_server_error;
             pub use checkout_core::server::Server;
             pub use checkout_core::internal_context::InternalContext;
+            pub use checkout_core::db::DBClient;
 
             use std::env;
             use tokio::task;
@@ -114,8 +115,9 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
             use checkout_core::server::CreateParams;
             use checkout_core::server::AddOrRemoveItemParams;
             use checkout_core::transaction::TransactionController;
-            use checkout_core::context::Context;
-            use checkout_core::context::ContextProvider;
+            use checkout_core::context::{Context, ClientContext};
+            use checkout_core::context::ClientContextProvider;
+            use checkout_core::master_context::MasterContext;
             use checkout_core::checkout::handlers::{create_handler, add_item_handler, remove_item_handler, confirm_items_handler, update_contact_handler, update_shipping_address_handler, update_fulfillment_type_handler, initiate_payment_handler, initiate_order_handler};
 
             pub struct Config {
@@ -134,9 +136,29 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                 }
             }
 
+            // begin addition
+
+            pub struct MasterContextProvider {
+                db_client: DBClient,
+                client_context_provider: super::#server_type_name,
+            }
+
+            impl MasterContextProvider {
+                pub fn new(db_client: DBClient, client_context_provider: super::#server_type_name) -> Self {
+                    Self { db_client, client_context_provider }
+                }
+
+                pub async fn new_context<#common_lifetime>(&#common_lifetime self) -> Result<MasterContext<#common_lifetime, #context_type_name #ty_generics>, Error> {
+                    let client_ctx = self.client_context_provider.new_context().await?;
+                    Ok(MasterContext::new(client_ctx, &self.db_client))
+                }
+            }
+
+            // end addition
+
             pub struct ServerContext {
                 pub internal_context: InternalContext,
-                pub context_provider: Arc<super::#server_type_name>,
+                pub context_provider: Arc<MasterContextProvider>,
             }
 
             #[derive(Debug)]
@@ -156,7 +178,7 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
 
             pub fn start_payment_processing(
                 internal_ctx: InternalContext,
-                context_provider: Arc<super::#server_type_name>,
+                context_provider: Arc<MasterContextProvider>,
             ) {
                 task::spawn(async move {
                     let subscription = internal_ctx.payment_processing_subscription.clone();
@@ -740,7 +762,12 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                 let api = Self::api_description();
                 let config = checkout_server::get_config();
                 let internal_context = checkout_server::InternalContext::new(config.google_application_credentials.clone(), config.payment_processing_topic.clone(), config.order_events_topic.clone()).await?;
-                let context_provider = checkout_server::Arc::new(self);
+
+                let db_client = checkout_server::DBClient::new().await.map_err(|_| checkout_server::new_server_error("failed to create database client"))?;
+                let context_provider = checkout_server::Arc::new(checkout_server::MasterContextProvider::new(
+                    db_client,
+                    self,
+                ));
 
                 let pubsub_client = internal_context.pubsub_client.clone();
                 pubsub_client.spawn_token_renew(checkout_server::Duration::from_secs(15 * 60));
